"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var eventemitter2_1 = require("eventemitter2");
var json = require("../json");
var lodash = require("../lodash");
var auth_1 = require("../auth");
var database_1 = require("../database");
var firestore_1 = require("../firestore");
var messaging_1 = require("../messaging");
var storage_1 = require("../storage");
var MockApp = (function () {
    function MockApp(options) {
        this.deleter_ = options.deleter;
        this.initializeOptions_ = options.initializeOptions;
        this.name_ = options.name;
        this.auth_ = new auth_1.MockAuth({
            app: this,
            identities: options.identities
        });
        this.databaseEmitters_ = {
            root: new eventemitter2_1.EventEmitter2({ wildcard: true }),
            shared: {}
        };
        this.databaseEmitters_.root.onAny(this.databaseRootListener_.bind(this));
        this.database_ = new database_1.MockDatabase({
            app: this,
            database: options.database || { content: null },
            emitters: this.databaseEmitters_
        });
        this.firestoreEmitters_ = {
            root: new eventemitter2_1.EventEmitter2({ wildcard: true }),
            shared: {}
        };
        this.firestoreEmitters_.root.onAny(this.firestoreRootListener_.bind(this));
        this.firestore_ = new firestore_1.MockFirestore({
            app: this,
            emitters: this.firestoreEmitters_,
            firestore: options.firestore || {
                content: null,
                fieldPath: {},
                fieldValue: {}
            }
        });
        this.messaging_ = new messaging_1.MockMessaging({
            app: this
        });
    }
    Object.defineProperty(MockApp.prototype, "name", {
        get: function () {
            return this.name_;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockApp.prototype, "options", {
        get: function () {
            return this.initializeOptions_;
        },
        enumerable: true,
        configurable: true
    });
    MockApp.prototype.auth = function () {
        return this.auth_;
    };
    MockApp.prototype.database = function () {
        return this.database_;
    };
    MockApp.prototype.delete = function () {
        return this.deleter_();
    };
    MockApp.prototype.firestore = function () {
        return this.firestore_;
    };
    MockApp.prototype.messaging = function () {
        return this.messaging_;
    };
    MockApp.prototype.storage = function (url) {
        return new storage_1.MockStorage({
            app: this,
            url: url
        });
    };
    MockApp.prototype.databaseRootListener_ = function (eventType, _a) {
        var _this = this;
        var content = _a.content, previousContent = _a.previousContent;
        lodash.each(this.databaseEmitters_.shared, function (sharedEmitter, sharedEmitterJsonPath) {
            var sharedEmitterRef = _this.database().ref(lodash.trim(sharedEmitterJsonPath, "/"));
            var value = null;
            if (content && json.has(content, sharedEmitterJsonPath)) {
                value = json.get(content, sharedEmitterJsonPath);
            }
            var previousValue = null;
            if (previousContent && json.has(previousContent, sharedEmitterJsonPath)) {
                previousValue = json.get(previousContent, sharedEmitterJsonPath);
            }
            if (value !== previousValue) {
                sharedEmitter.emit("value", {
                    previousSnapshot: null,
                    snapshot: new database_1.MockDataSnapshot({
                        content: content,
                        ref: sharedEmitterRef
                    })
                });
            }
            if (value === null) {
                value = {};
            }
            if (previousValue === null) {
                previousValue = {};
            }
            if ((typeof value === "object") && (typeof previousValue === "object")) {
                var childKeys = Object.keys(value);
                var previousChildKeys = Object.keys(previousValue);
                lodash.each(lodash.differenceWith(previousChildKeys, childKeys), function (removedKey) {
                    sharedEmitter.emit("child_removed", {
                        previousSnapshot: new database_1.MockDataSnapshot({
                            content: previousContent,
                            ref: sharedEmitterRef.child(removedKey)
                        }),
                        snapshot: new database_1.MockDataSnapshot({
                            content: content,
                            ref: sharedEmitterRef.child(removedKey)
                        })
                    });
                });
                lodash.each(lodash.differenceWith(childKeys, previousChildKeys), function (addedKey) {
                    sharedEmitter.emit("child_added", {
                        previousSnapshot: new database_1.MockDataSnapshot({
                            content: previousContent,
                            ref: sharedEmitterRef.child(addedKey)
                        }),
                        snapshot: new database_1.MockDataSnapshot({
                            content: content,
                            ref: sharedEmitterRef.child(addedKey)
                        })
                    });
                });
                lodash.each(lodash.intersectionWith(previousChildKeys, childKeys), function (changedKey) {
                    if (value[changedKey] !== previousValue[changedKey]) {
                        sharedEmitter.emit("child_changed", {
                            previousSnapshot: new database_1.MockDataSnapshot({
                                content: previousContent,
                                ref: sharedEmitterRef.child(changedKey)
                            }),
                            snapshot: new database_1.MockDataSnapshot({
                                content: content,
                                ref: sharedEmitterRef.child(changedKey)
                            })
                        });
                    }
                });
            }
        });
    };
    MockApp.prototype.firestoreRootListener_ = function (eventType, _a) {
        var content = _a.content, previousContent = _a.previousContent;
        lodash.each(this.firestoreEmitters_.shared, function (sharedEmitter, sharedEmitterJsonPath) {
            sharedEmitter.emit("content", { content: content, previousContent: previousContent });
        });
    };
    return MockApp;
}());
exports.MockApp = MockApp;
